@@ -35,6 +35,7 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <string.h>
+#include <math.h>
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
@@ -97,6 +98,10 @@ struct _GSRWindowPrivate {
 	GtkWidget *rate, *time_sec, *format, *channels;
 	GtkWidget *name_label;
 	GtkWidget *length_label;
+	GtkWidget *align;
+	GtkWidget *volume_label;
+	GtkWidget *level;
+
 	gulong seek_id;
 
 	GtkUIManager *ui_manager;
@@ -148,7 +153,7 @@ struct _GSRWindowPrivate {
 };
 
 static gboolean            make_record_source      (GSRWindow         *window);
-static void                fill_record_input       (GSRWindow         *window);
+static void                fill_record_input       (GSRWindow         *window, gchar *selected);
 static GSRWindowPipeline * make_record_pipeline    (GSRWindow         *window);
 static GSRWindowPipeline * make_play_pipeline      (GSRWindow         *window);
 
@@ -278,57 +283,65 @@ seconds_to_string (guint seconds)
 static char *
 seconds_to_full_string (guint seconds)
 {
-	int hour, min, sec;
+	long days, hours, minutes;
+	char *time = NULL;
+	const char *minutefmt;
+	const char *hourfmt;
+	const char *dayfmt;
+	const char *secondfmt;
 
-	min = (seconds / 60);
-	hour = (min / 60);
-	min -= (hour * 60);
-	sec = seconds - ((hour * 3600) + (min * 60));
+	days    = seconds / (60 * 60 * 24);
+	hours   = (seconds / (60 * 60));
+	minutes = (seconds / 60) - ((days * 24 * 60) + (hours * 60));
+	seconds = seconds % 60;
 
-	if (hour > 0) {
-		if (min > 0) {
-			if (sec > 0) {
-				return g_strdup_printf ("%d %s %d %s %d %s",
-							hour, hour > 1 ? _("hours") : _("hour"),
-							min, min > 1 ? _("minutes") : _("minute"),
-							sec, sec > 1 ? _("seconds") : _("second"));
+	minutefmt = ngettext ("%ld minute", "%ld minutes", minutes);
+	hourfmt = ngettext ("%ld hour", "%ld hours", hours);
+	secondfmt = ngettext ("%ld second", "%ld seconds", seconds);
+
+	if (hours > 0) {
+		if (minutes > 0)
+			if (seconds > 0) {
+				char *fmt;
+				/* Translators: the format is "X hours, X minutes and X seconds" */
+				fmt = g_strdup_printf (_("%s, %s and %s"), hourfmt, minutefmt, secondfmt);
+				time = g_strdup_printf (fmt, hours, minutes, seconds);
+				g_free (fmt);
 			} else {
-				return g_strdup_printf ("%d %s %d %s",
-							hour, hour > 1 ? _("hours") : _("hour"),
-							min, min > 1 ? _("minutes") : _("minute"));
+				char *fmt;
+				/* Translators: the format is "X hours and X minutes" */
+				fmt = g_strdup_printf (_("%s and %s"), hourfmt, minutefmt);
+				time = g_strdup_printf (fmt, hours, minutes);
+				g_free (fmt);
 			}
-		} else {
-			if (sec > 0) {
-				return g_strdup_printf ("%d %s %d %s",
-							hour, hour > 1 ? _("hours") : _("hour"),
-							sec, sec > 1 ? _("seconds") : _("second"));
+		else
+			if (seconds > 0) {
+				char *fmt;
+				/* Translators: the format is "X minutes and X seconds" */
+				fmt = g_strdup_printf (_("%s and %s"), minutefmt, secondfmt);
+				time = g_strdup_printf (fmt, minutes, seconds);
+				g_free (fmt);
 			} else {
-				return g_strdup_printf ("%d %s",
-							hour, hour > 1 ? _("hours") : _("hour"));
+				time = g_strdup_printf (minutefmt, minutes);
 			}
-		}
 	} else {
-		if (min > 0) {
-			if (sec > 0) {
-				return g_strdup_printf ("%d %s %02d %s",
-							min, min > 1 ? _("minutes") : _("minute"),
-							sec, sec > 1 ? _("seconds") : _("second"));
+		if (minutes > 0) {
+			if (seconds > 0) {
+				char *fmt;
+				/* Translators: the format is "X minutes and X seconds" */
+				fmt = g_strdup_printf (_("%s and %s"), minutefmt, secondfmt);
+				time = g_strdup_printf (fmt, minutes, seconds);
+				g_free (fmt);
 			} else {
-				return g_strdup_printf ("%d %s",
-							min, min > 1 ? _("minutes") : _("minute"));
+				time = g_strdup_printf (minutefmt, minutes);
 			}
+
 		} else {
-			if (sec == 0) {
-				return g_strdup_printf ("%d %s",
-							sec, _("seconds"));
-			} else {
-				return g_strdup_printf ("%d %s",
-							sec, sec > 1 ? _("seconds") : _("second"));
-			}
+			time = g_strdup_printf (secondfmt, seconds);
 		}
 	}
 
-	return NULL;
+	return time;
 }
 
 static void
@@ -354,6 +367,7 @@ file_open_cb (GtkAction *action,
 {
 	GtkWidget *file_chooser;
 	gchar *directory;
+	gchar *locale_directory = NULL;
 	gint response;
 
 	g_return_if_fail (GSR_IS_WINDOW (window));
@@ -368,22 +382,29 @@ file_open_cb (GtkAction *action,
 	directory = gconf_client_get_string (gconf_client, KEY_OPEN_DIR, NULL);
 
 	if (directory != NULL && *directory != 0) {
+		locale_directory = g_filename_from_utf8 (directory, -1, NULL, NULL, NULL);
+		if (!locale_directory || !g_file_test (locale_directory, G_FILE_TEST_EXISTS))
+			locale_directory = g_strdup (directory);
 		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (file_chooser),
-		                                     directory);
+		                                     locale_directory);
+		g_free (locale_directory);
 	}
 
 	response = gtk_dialog_run (GTK_DIALOG (file_chooser));
 
 	if (response == GTK_RESPONSE_OK) {
 		gchar *name;
+		gchar *utf8_name = NULL;
 
 		name = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_chooser));
 		if (name) {
 			gchar *dirname;
 
-			dirname = g_path_get_dirname (name);
+			utf8_name = g_filename_to_utf8 (name, -1, NULL, NULL, NULL);
+			dirname = g_path_get_dirname (utf8_name);
 			gconf_client_set_string (gconf_client, KEY_OPEN_DIR, dirname, NULL);
 			g_free (dirname);
+			g_free (utf8_name);
 
 			if (window->priv->has_file == TRUE) {
 				/* Just open a new window with the file */
@@ -391,6 +412,7 @@ file_open_cb (GtkAction *action,
 			} else {
 				/* Set the file in this window */
 				g_object_set (G_OBJECT (window), "location", name, NULL);
+				window->priv->dirty = FALSE;
 			}
 		
 			g_free (name);
@@ -446,6 +468,7 @@ file_open_recent_cb (EggRecentViewGtk *view,
 	} else {
 		/* Set the file in this window */
 		g_object_set (G_OBJECT (window), "location", filename, NULL);
+		window->priv->dirty = FALSE;
 	}
 
  out:
@@ -654,14 +677,12 @@ do_save_file (GSRWindow *window,
 	      const char *_name)
 {
 	GSRWindowPrivate *priv;
-        GMAudioProfile *profile;
 	char *tmp, *src, *name;
+	gchar *utf8_name = NULL;
 	GnomeVFSURI *src_uri, *dst_uri;
 
 	priv = window->priv;
 
-        profile = gm_audio_profile_choose_get_active (priv->profile);
-
 	if (window->priv->extension == NULL ||
 	    g_str_has_suffix (_name, window->priv->extension))
 		name = g_strdup (_name);
@@ -669,12 +690,16 @@ do_save_file (GSRWindow *window,
 		name = g_strdup_printf ("%s.%s", _name,
 				window->priv->extension);
 	if (g_file_test (name, G_FILE_TEST_EXISTS)) {
-		if (!replace_existing_file (GTK_WINDOW (window), name))
+		utf8_name = g_filename_to_utf8 (name, -1, NULL, NULL, NULL);
+		if (!replace_existing_file (GTK_WINDOW (window), utf8_name)) {
+			g_free (utf8_name);
 			return;
+		}
+		g_free (utf8_name);
 	}
 
-	tmp = g_strdup_printf ("file://%s", name);
-	src = g_strdup_printf ("file://%s", priv->record_filename);
+	tmp = gnome_vfs_make_uri_from_shell_arg (name);
+	src = gnome_vfs_make_uri_from_shell_arg (priv->record_filename);
 	src_uri = gnome_vfs_uri_new (src);
 	dst_uri = gnome_vfs_uri_new (tmp);
 	g_free (src);
@@ -700,8 +725,10 @@ do_save_file (GSRWindow *window,
 		gnome_vfs_uri_unref (src_uri);
 		gnome_vfs_uri_unref (dst_uri);
 	} else {
+		utf8_name = g_filename_to_utf8 (name, -1, NULL, NULL, NULL);
 		show_error_dialog (GTK_WINDOW (window), NULL,
-			           _("Could not save the file \"%s\""), name);
+			           _("Could not save the file \"%s\""), utf8_name);
+		g_free (utf8_name);
 	}
 
 	g_free (name);
@@ -714,6 +741,7 @@ file_save_as_cb (GtkAction *action,
 {
 	GtkWidget *file_chooser;
 	gchar *directory;
+	gchar *locale_directory = NULL;
 	gint response;
 
 	g_return_if_fail (GSR_IS_WINDOW (window));
@@ -727,26 +755,30 @@ file_save_as_cb (GtkAction *action,
 
 	directory = gconf_client_get_string (gconf_client, KEY_SAVE_DIR, NULL);
 	if (directory != NULL && *directory != 0) {
-		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (file_chooser), directory);
+		locale_directory = g_filename_from_utf8 (directory, -1, NULL, NULL, NULL);
+		if (!locale_directory || !g_file_test (locale_directory, G_FILE_TEST_EXISTS))
+			locale_directory = g_strdup (directory);
+		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (file_chooser), locale_directory);
+		g_free (locale_directory);
 	}
 	g_free (directory);
 
 	if (window->priv->filename != NULL) {
-		char *basename;
+		char *locale_basename;
+		char *basename = NULL;
 		gchar *filename, *filename_ext, *extension;
 		gint length;
-		GMAudioProfile *profile;
 
-		basename = g_path_get_basename (window->priv->filename);
+		locale_basename = g_path_get_basename (window->priv->filename);
+		basename = g_filename_to_utf8 (locale_basename, -1, NULL, NULL, NULL);
 		length = strlen (basename);
-		extension = strrchr (basename, '.');
+		extension = g_strrstr (basename, ".");
 
 		if (extension != NULL) {
 			length = length - strlen (extension);
 		}
 
 		filename = g_strndup (basename,length);
-		profile = gm_audio_profile_choose_get_active (window->priv->profile);
 		if (window->priv->extension)
 			filename_ext = g_strdup_printf ("%s.%s", filename,
 						window->priv->extension);
@@ -757,20 +789,24 @@ file_save_as_cb (GtkAction *action,
 		g_free (filename);
 		g_free (filename_ext);
 		g_free (basename);
+		g_free (locale_basename);
 	}
 
 	response = gtk_dialog_run (GTK_DIALOG (file_chooser));
 
 	if (response == GTK_RESPONSE_OK) {
 		gchar *name;
+		gchar *utf8_name = NULL;
 
 		name = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_chooser));
 		if (name) {
 			gchar *dirname;
 
-			dirname = g_path_get_dirname (name);
+			utf8_name= g_filename_to_utf8 (name, -1, NULL, NULL, NULL);
+			dirname = g_path_get_dirname (utf8_name);
 			gconf_client_set_string (gconf_client, KEY_SAVE_DIR, dirname, NULL);
 			g_free (dirname);
+			g_free (utf8_name);
 	
 			do_save_file (window, name);
 			g_free (name);
@@ -780,12 +816,11 @@ file_save_as_cb (GtkAction *action,
 	gtk_widget_destroy (file_chooser);
 }
 
-
 static void
 file_save_cb (GtkAction *action,
 	      GSRWindow *window)
 {
-	if (window->priv->dirty) {
+	if (!window->priv->has_file) {
 		file_save_as_cb (NULL, window);
 	} else {
 		do_save_file (window, window->priv->filename);
@@ -797,7 +832,7 @@ run_mixer_cb (GtkAction *action,
 	       GSRWindow *window)
 {
 	char *mixer_path;
-	char *argv[2] = {NULL, NULL};
+	char *argv[4] = {NULL, "--page", "recording",  NULL};
 	GError *error = NULL;
 	gboolean ret;
 
@@ -835,9 +870,16 @@ close_confirmation_dialog (GSRWindow *window)
 	AtkObject *atk_obj;
 	gchar *msg;
 	gint response_id;
+	gchar *utf8_filename = NULL;
 
+	utf8_filename = g_filename_to_utf8 (window->priv->record_filename,
+	                                    -1,
+	                                    NULL,
+	                                    NULL,
+	                                    NULL);
 	msg = g_strdup_printf (_("Save the changes to file \"%s\" before closing?"),
-			       window->priv->record_filename);
+			       utf8_filename);
+	g_free (utf8_filename);
 
 	confirmation_dialog = gtk_message_dialog_new_with_markup (NULL,
 								  GTK_DIALOG_MODAL,
@@ -936,22 +978,32 @@ fill_in_information (GSRWindow *window,
 	struct stat buf;
 	guint64 file_size = 0;
 	gchar *text, *name;
+	gchar *utf8_name = NULL;
 	gint n_channels, bitrate, samplerate;
 
 	/* dirname */
-	text = g_path_get_dirname (window->priv->filename);
-	gtk_label_set_text (GTK_LABEL (fp->dirname), text);
-	g_free (text);
+	if (window->priv->dirty) {
+		gtk_label_set_text (GTK_LABEL (fp->dirname), "");
+	} else {
+		name = g_path_get_dirname (window->priv->filename);
+		text = g_filename_to_utf8 (name, -1, NULL, NULL, NULL);
+		gtk_label_set_text (GTK_LABEL (fp->dirname), text);
+		g_free (text);
+		g_free (name);
+	}
 
 	/* filename */
 	name = g_path_get_basename (window->priv->filename);
+	utf8_name = g_filename_to_utf8 (name, -1, NULL, NULL, NULL);
+
 	if (window->priv->dirty) {
-		text = g_strdup_printf (_("%s (Has not been saved)"), name);
+		text = g_strdup_printf (_("%s (Has not been saved)"), utf8_name);
 	} else {
-		text = g_strdup (name);
+		text = g_strdup (utf8_name);
 	}
 	gtk_label_set_text (GTK_LABEL (fp->filename), text);
 	g_free (text);
+	g_free (utf8_name);
 	g_free (name);
 	
 	/* Size */
@@ -1203,7 +1255,7 @@ about_cb (GtkAction *action,
 			       "comments", _("A sound recorder for GNOME\n gnome-media@gnome.org"),
 			       "authors", authors,
 			       "documenters", documenters,
-			       "logo-icon-name", "gnome-grecord",
+			       "logo-icon-name", "gnome-sound-recorder",
 			       NULL);
 }
 
@@ -1213,7 +1265,7 @@ play_cb (GtkAction *action,
 {
 	GSRWindowPrivate *priv = window->priv;
 
-	if (priv->has_file == FALSE)
+	if (priv->has_file == FALSE && !priv->working_file)
 		return;
 
 	if (priv->play) {
@@ -1222,14 +1274,12 @@ play_cb (GtkAction *action,
 
 	if ((priv->play = make_play_pipeline (window))) {
 		gchar *uri;
+		gchar *usefile;
 
-		/* FIXME: do we need to complete relative paths
-		 * here or is it always an absolute path? */
-		if (!g_path_is_absolute (priv->filename)) {
-			g_warning ("Filename '%s' is not an absolute path (FIXME)",
-			           priv->filename);
-		}
-		uri = g_strdup_printf ("file://%s", priv->filename);
+		if(priv->has_file == FALSE && priv->working_file) usefile = priv->working_file;
+		else usefile = priv->filename;
+
+		uri = gnome_vfs_make_uri_from_shell_arg (usefile);
 		g_object_set (window->priv->play->pipeline, "uri", uri, NULL);
 		g_free (uri);
 
@@ -1260,6 +1310,9 @@ stop_cb (GtkAction *action,
 
 		GST_DEBUG ("Stopping recording pipeline");
 		set_pipeline_state_to_null (priv->record->pipeline);
+		gtk_widget_set_sensitive (window->priv->level, FALSE);
+		gtk_widget_set_sensitive (window->priv->volume_label, FALSE);
+		gtk_progress_set_percentage (GTK_PROGRESS (window->priv->level), 0);
 	}
 }
 
@@ -1270,20 +1323,26 @@ record_cb (GtkAction *action,
 	GSRWindowPrivate *priv = window->priv;
 
 	if (priv->record) {
+		char *current_source;
 		shutdown_pipeline (priv->record);
 		if (!make_record_source (window)) exit (1);
-		fill_record_input (window);
+		current_source = gtk_combo_box_get_active_text (GTK_COMBO_BOX (window->priv->input));
+		fill_record_input (window, current_source);
 	}
 
 	if ((priv->record = make_record_pipeline (window))) {
 		window->priv->len_secs = 0;
 		window->priv->saved = FALSE;
 
+		g_print (priv->record_filename);
 		g_object_set (G_OBJECT (priv->record->sink),
 			      "location", priv->record_filename,
 			      NULL);
 
 		gst_element_set_state (priv->record->pipeline, GST_STATE_PLAYING);
+		gtk_widget_set_sensitive (window->priv->level, TRUE);
+		gtk_widget_set_sensitive (window->priv->volume_label, TRUE);
+
 	}
 }
 
@@ -1556,12 +1615,15 @@ profile_changed_cb (GObject *object, GSRWindow *window)
 	gchar *id;
 
 	g_return_if_fail (GTK_IS_COMBO_BOX (object));
+
 	profile = gm_audio_profile_choose_get_active (GTK_WIDGET (object));
-  
-	id = g_strdup (gm_audio_profile_get_id (profile));
-	GST_DEBUG ("profile changed to %s", GST_STR_NULL (id));
-	gconf_client_set_string (gconf_client, KEY_LAST_PROFILE_ID, id, NULL);
-	g_free (id);
+
+	if (profile != NULL) {
+		id = g_strdup (gm_audio_profile_get_id (profile));
+		GST_DEBUG ("profile changed to %s", GST_STR_NULL (id));
+		gconf_client_set_string (gconf_client, KEY_LAST_PROFILE_ID, id, NULL);
+		g_free (id);
+	}
 }
 
 static void
@@ -1645,7 +1707,6 @@ record_eos_msg_cb (GstBus * bus, GstMessage * msg, GSRWindow * window)
 	g_free (window->priv->filename);
 	window->priv->filename = g_strdup (window->priv->record_filename);
 
-	window->priv->dirty = TRUE;
 	window->priv->has_file = TRUE;
 }
 
@@ -1658,7 +1719,7 @@ record_start (gpointer user_data)
 	gchar *name;
 
 	window->priv->get_length_attempts = 16;
-	g_timeout_add (200, (GSourceFunc) record_tick_callback, window);
+	window->priv->tick_id = g_timeout_add (200, (GSourceFunc) record_tick_callback, window);
 
 	set_action_sensitive (window, "Stop", TRUE);
 	set_action_sensitive (window, "Play", FALSE);
@@ -1715,7 +1776,7 @@ record_state_changed_cb (GstBus *bus, GstMessage *msg, GSRWindow *window)
 		window->priv->record_id = g_idle_add (record_start, window);
 		g_free (window->priv->extension);
 		profile = gm_audio_profile_choose_get_active (window->priv->profile);
-		window->priv->extension = g_strdup (gm_audio_profile_get_extension (profile));
+		window->priv->extension = g_strdup (profile ? gm_audio_profile_get_extension (profile) : NULL);
 		gtk_widget_set_sensitive (window->priv->profile, FALSE);
 		gtk_widget_set_sensitive (window->priv->input, FALSE);
 		break;
@@ -1723,7 +1784,7 @@ record_state_changed_cb (GstBus *bus, GstMessage *msg, GSRWindow *window)
 		gtk_adjustment_set_value (GTK_RANGE (window->priv->scale)->adjustment, 0.0);
 		gtk_widget_set_sensitive (window->priv->scale, FALSE);
 		gtk_widget_set_sensitive (window->priv->profile, TRUE);
-		gtk_widget_set_sensitive (window->priv->input, TRUE);
+		gtk_widget_set_sensitive (window->priv->input, GST_IS_MIXER (window->priv->mixer));
 		/* fall through */
 	case GST_STATE_PAUSED:
 		set_action_sensitive (window, "Stop", FALSE);
@@ -1738,6 +1799,10 @@ record_state_changed_cb (GstBus *bus, GstMessage *msg, GSRWindow *window)
 		gtk_statusbar_push (GTK_STATUSBAR (window->priv->statusbar),
 				    window->priv->status_message_cid,
 				    _("Ready"));
+		if (window->priv->tick_id > 0) {
+			g_source_remove (window->priv->tick_id);
+			window->priv->tick_id = 0;
+		}
 		break;
 	default:
 		break;
@@ -1765,7 +1830,8 @@ make_record_source (GSRWindow *window)
         if (!gst_element_set_state (source, GST_STATE_READY)) {
 		show_error_dialog (NULL, NULL,
 			_("Your audio capture settings are invalid. "
-			  "Please correct them in the Multimedia settings."));
+			  "Please correct them with the \"Sound Preferences\" "
+			  "under the System-Preferences menu."));
 		return FALSE;
 	}
 	window->priv->source = source;
@@ -1786,19 +1852,36 @@ record_input_changed_cb (GtkComboBox *input, GSRWindow *window)
 	text = gtk_combo_box_get_active_text (input);
 	GST_DEBUG ("record input changed to '%s'", GST_STR_NULL (text));
 
+	if (text == NULL)
+		return;
+
+	/* The pipeline has been destroyed already, we'll try and remember
+	 * the input for the next record run in fill_record_input() */
+	if (GST_IS_MIXER (window->priv->mixer) == FALSE)
+		return;
+
 	for (l = gst_mixer_list_tracks (window->priv->mixer);
 	     l != NULL; l = l->next) {
 		t = l->data;
-		if ((strcmp (t->label, text) == 0) &&
+		if (t == NULL || t->label == NULL)
+			continue;
+		if ((g_str_equal (t->label, text)) &&
 		    (t->flags & GST_MIXER_TRACK_INPUT)) {
 			if (new == NULL)
 				new = g_object_ref (t);
+		/* FIXME selected == t is equivalent to NULL == t in this case,
+		 * selected, after its initialization to NULL, was never written to
+		 * before this read access to it
+		 * and NULL == t is equivalent to FALSE, because of the check
+		 * "if (t == NULL || t->label == NULL)" above
+		 */
 		} else if (selected == t)
 			/* re-mute old one */
 			gst_mixer_set_record (window->priv->mixer,
 					      selected, FALSE);
 	}
 
+	/* FIXME selected _is_ NULL always at this point - same as 5 lines above*/
 	if (selected != NULL)
 		g_object_unref (selected);
 	if (!(selected = new))
@@ -1809,7 +1892,7 @@ record_input_changed_cb (GtkComboBox *input, GSRWindow *window)
 }
 
 static void
-fill_record_input (GSRWindow *window)
+fill_record_input (GSRWindow *window, gchar *selected)
 {
 	GstElement *e;
 	const GList *l;
@@ -1821,27 +1904,76 @@ fill_record_input (GSRWindow *window)
 	if (model) 
 		gtk_list_store_clear (GTK_LIST_STORE (model));
 	
-	g_return_if_fail (GST_IS_MIXER (window->priv->mixer));
+	gtk_widget_set_sensitive (window->priv->input, GST_IS_MIXER (window->priv->mixer));
+	if (!GST_IS_MIXER (window->priv->mixer))
+		return;
 
 	for (l = gst_mixer_list_tracks (window->priv->mixer); l != NULL; l = l->next) {
 		GstMixerTrack *t = l->data;
+		if (t->label == NULL)
+			continue;
 		if (t->flags & GST_MIXER_TRACK_INPUT) {
 			gtk_combo_box_append_text (GTK_COMBO_BOX (window->priv->input), t->label);
 			++i;
 		}
-		if (t->flags & GST_MIXER_TRACK_RECORD) {
+		if ((t->flags & GST_MIXER_TRACK_RECORD) && (selected == NULL)) {
+			gtk_combo_box_set_active (GTK_COMBO_BOX (window->priv->input), i - 1);
+		}
+		if ((selected != NULL) && g_str_equal (selected, t->label)) {
 			gtk_combo_box_set_active (GTK_COMBO_BOX (window->priv->input), i - 1);
 		}
 	}
 }
 
+gboolean
+level_message_handler_cb (GstBus * bus, GstMessage * message, GSRWindow *window)
+{
+  GSRWindowPrivate *priv;
+  GstState cur_state, pending;
+
+  priv = window->priv;
+
+  if (message->type == GST_MESSAGE_ELEMENT) {
+    const GstStructure *s = gst_message_get_structure (message);
+    const gchar *name = gst_structure_get_name (s);
+
+    if (g_str_equal (name, "level")) {
+      gint channels;
+      gdouble peak_dB;
+      gdouble myind;
+      const GValue *list;
+      const GValue *value;
+
+      gint i;
+      /* we can get the number of channels as the length of any of the value
+       * lists */
+
+      list = gst_structure_get_value (s, "rms");
+      channels = gst_value_list_get_size (list);
+
+      for (i = 0; i < channels; ++i) {
+	list = gst_structure_get_value (s, "peak");
+        value = gst_value_list_get_value (list, i);
+        peak_dB = g_value_get_double (value);
+	myind = exp (peak_dB / 20);
+	if (myind > 1.0)
+		myind = 1.0;
+	gtk_progress_set_percentage (GTK_PROGRESS (window->priv->level), myind);
+      }
+    }
+  }
+  /* we handled the message we want, and ignored the ones we didn't want.
+   * so the core can unref the message for us */
+  return TRUE;
+}
+
 static GSRWindowPipeline *
 make_record_pipeline (GSRWindow *window)
 {
 	GSRWindowPipeline *pipeline;
 	GMAudioProfile *profile;
 	const gchar *profile_pipeline_desc;
-	GstElement *encoder, *source, *filesink;
+	GstElement *encoder, *source, *filesink, *level;
 	GError *err = NULL;
 	gchar *pipeline_desc;
 	const char *name;
@@ -1867,7 +1999,20 @@ make_record_pipeline (GSRWindow *window)
 
 	gst_bin_add (GST_BIN (pipeline->pipeline), source);
 
+	level = gst_element_factory_make ("level", "level");
+	if (level == NULL)
+	{
+		show_missing_known_element_error (NULL,
+		    _("level"), "level", "level",
+		    "gstreamer");
+		gst_object_unref (source);
+		return NULL;
+	}
+	gst_element_set_name (level, "level");
+
 	profile = gm_audio_profile_choose_get_active (window->priv->profile);
+	if (profile == NULL)
+		return NULL;
 	profile_pipeline_desc = gm_audio_profile_get_pipeline (profile);
 	name = gm_audio_profile_get_name (profile);
 
@@ -1892,11 +2037,12 @@ make_record_pipeline (GSRWindow *window)
 		return NULL;
 	}
 
+	gst_bin_add (GST_BIN (pipeline->pipeline), level);
 	gst_bin_add (GST_BIN (pipeline->pipeline), encoder);
 	gst_bin_add (GST_BIN (pipeline->pipeline), filesink);
 
 	/* now link it all together */
-	if (!gst_element_link (source, encoder)) {
+	if (!(gst_element_link_many (source, level, encoder, NULL))) {
 		show_profile_error (NULL, err->message,
 			_("Could not capture using the '%s' audio profile. "),
 			name);
@@ -1925,6 +2071,10 @@ make_record_pipeline (GSRWindow *window)
 
 	gst_bus_add_signal_watch (pipeline->bus);
 
+	g_signal_connect (pipeline->bus, "message::element",
+	                  G_CALLBACK (level_message_handler_cb),
+	                  window);
+
 	g_signal_connect (pipeline->bus, "message::state-changed",
 	                  G_CALLBACK (record_state_changed_cb),
 	                  window);
@@ -1962,11 +2112,11 @@ static const GtkActionEntry menu_entries[] =
 	{ "File", NULL, N_("_File") },
 	{ "FileNew", GTK_STOCK_NEW, NULL, NULL,
 	  N_("Create a new sample"), G_CALLBACK (file_new_cb) },
-	{ "FileOpen", GTK_STOCK_OPEN, N_("_Open..."), NULL,
+	{ "FileOpen", GTK_STOCK_OPEN, NULL, NULL,
 	  N_("Open a file"), G_CALLBACK (file_open_cb) },
 	{ "FileSave", GTK_STOCK_SAVE, NULL, NULL,
 	  N_("Save the current file"), G_CALLBACK (file_save_cb) },
-	{ "FileSaveAs", GTK_STOCK_SAVE_AS, N_("Save _As..."), "<shift><control>S",
+	{ "FileSaveAs", GTK_STOCK_SAVE_AS, NULL, "<shift><control>S",
 	  N_("Save the current file with a different name"), G_CALLBACK (file_save_as_cb) },
 	{ "RunMixer", GTK_STOCK_EXECUTE, N_("Open Volu_me Control"), NULL,
 	  N_("Open the audio mixer"), G_CALLBACK (run_mixer_cb) },
@@ -1988,7 +2138,7 @@ static const GtkActionEntry menu_entries[] =
 
 	/* Help menu */
 	{ "Help", NULL, N_("_Help") },
-	{"HelpContents", GTK_STOCK_HELP, NULL, NULL,
+	{"HelpContents", GTK_STOCK_HELP, N_("Contents"), "F1",
 	 N_("Open the manual"), G_CALLBACK (help_contents_cb) },
 	{ "About", GTK_STOCK_ABOUT, NULL, NULL,
 	 N_("About this application"), G_CALLBACK (about_cb) }
@@ -2082,6 +2232,7 @@ gsr_window_init (GSRWindow *window)
 	GtkWidget *hbox;
 	GtkWidget *label;
 	GtkWidget *table;
+	GtkWidget *align;
 	gchar *id;
 	gchar *path;
 	GtkAction *action;
@@ -2187,23 +2338,24 @@ gsr_window_init (GSRWindow *window)
 	gtk_scale_set_value_pos (GTK_SCALE (window->priv->scale), GTK_POS_BOTTOM);
 	/* We can't seek until we find out the length */
 	gtk_widget_set_sensitive (window->priv->scale, FALSE);
-	gtk_box_pack_start (GTK_BOX (content_vbox), priv->scale, TRUE, TRUE, 6);
+	gtk_box_pack_start (GTK_BOX (content_vbox), priv->scale, FALSE, FALSE, 6);
 	gtk_widget_show (window->priv->scale);
 
         /* create source and choose mixer input */
 	hbox = gtk_hbox_new (FALSE, 12);
 	gtk_box_pack_start (GTK_BOX (content_vbox), hbox, FALSE, FALSE, 0);
 
-	label = gtk_label_new (_("Record from input:"));
+	label = gtk_label_new_with_mnemonic (_("Record from _input:"));
 	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 
 	priv->input = gtk_combo_box_new_text ();
+	gtk_label_set_mnemonic_widget (GTK_LABEL (label), priv->input);
 	gtk_box_pack_start (GTK_BOX (hbox), priv->input, TRUE, TRUE, 0);
 	gtk_widget_show (priv->input);
 
 	if (!make_record_source (window)) exit (1);
-	fill_record_input (window);
+	fill_record_input (window, NULL);
 	g_signal_connect (priv->input, "changed",
 			  G_CALLBACK (record_input_changed_cb), window);
 
@@ -2215,11 +2367,12 @@ gsr_window_init (GSRWindow *window)
 	hbox = gtk_hbox_new (FALSE, 12);
 	gtk_box_pack_start (GTK_BOX (content_vbox), hbox, FALSE, FALSE, 0);
 
-	label = gtk_label_new (_("Record as:"));
+	label = gtk_label_new_with_mnemonic (_("_Record as:"));
 	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 
 	priv->profile = gm_audio_profile_choose_new ();
+	gtk_label_set_mnemonic_widget (GTK_LABEL (label), priv->profile);
 	gtk_box_pack_start (GTK_BOX (hbox), window->priv->profile, TRUE, TRUE, 0);
 	gtk_widget_show (window->priv->profile);
 
@@ -2236,34 +2389,38 @@ gsr_window_init (GSRWindow *window)
         g_signal_connect (priv->profile, "changed",
                           G_CALLBACK (profile_changed_cb), window);
 
-	label = make_title_label (_("File Information"));
-	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
-	gtk_box_pack_start (GTK_BOX (content_vbox), label, FALSE, FALSE, 0);
-
 	hbox = gtk_hbox_new (FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (content_vbox), hbox, FALSE, FALSE, 0);
 
-	label = gtk_label_new ("    ");
+	label = gtk_label_new ("    "); /* FIXME: better padding? */
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 
-	table = gtk_table_new (2, 2, FALSE);
+	table = gtk_table_new (3, 2, FALSE);
 	gtk_table_set_col_spacings (GTK_TABLE (table), 12);
 	gtk_table_set_row_spacings (GTK_TABLE (table), 6);
 	gtk_box_pack_start (GTK_BOX (hbox), table, TRUE, TRUE, 0);
 
-	label = gtk_label_new (_("Filename:"));
-	gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
+	label = make_title_label (_("File Information"));
+
+	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
 	gtk_table_attach (GTK_TABLE (table), label,
-			  0, 1, 0, 1,
-			  GTK_FILL, GTK_FILL, 0, 0);
+			  0, 2, 0, 1,
+			  GTK_FILL, 0, 0, 0);
+
+	label = gtk_label_new (_("Filename:"));
+
+	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+	gtk_table_attach (GTK_TABLE (table), label,
+			  0, 1, 1, 2,
+			  GTK_FILL, 0, 0, 0);
 
 	priv->name_label = gtk_label_new (_("<none>"));
 	gtk_label_set_selectable (GTK_LABEL (priv->name_label), TRUE);
 	gtk_label_set_line_wrap (GTK_LABEL (priv->name_label), GTK_WRAP_WORD);
 	gtk_misc_set_alignment (GTK_MISC (priv->name_label), 0, 0.5);
 	gtk_table_attach (GTK_TABLE (table), priv->name_label,
-			  1, 2, 0, 1,
-			  GTK_FILL, GTK_FILL,
+			  1, 2, 1, 2,
+			  GTK_FILL  | GTK_EXPAND, 0,
 			  0, 0);
 
 	atk_object_add_relationship (gtk_widget_get_accessible (GTK_WIDGET (priv->name_label)),
@@ -2272,17 +2429,18 @@ gsr_window_init (GSRWindow *window)
 
 
 	label = gtk_label_new (_("Length:"));
+
 	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
 	gtk_table_attach (GTK_TABLE (table), label,
-			  0, 1, 1, 2,
+			  0, 1, 2, 3,
 			  GTK_FILL, 0, 0, 0);
 	
 	priv->length_label = gtk_label_new ("");
 	gtk_label_set_selectable (GTK_LABEL (priv->length_label), TRUE);
 	gtk_misc_set_alignment (GTK_MISC (priv->length_label), 0, 0.5);
 	gtk_table_attach (GTK_TABLE (table), priv->length_label,
-			  1, 2, 1, 2,
-			  GTK_FILL, GTK_FILL,
+			  1, 2, 2, 3,
+			  GTK_FILL | GTK_EXPAND, 0,
 			  0, 0);
 
 	atk_object_add_relationship (gtk_widget_get_accessible (GTK_WIDGET (priv->length_label)),
@@ -2294,6 +2452,36 @@ gsr_window_init (GSRWindow *window)
 	GTK_WIDGET_SET_FLAGS (priv->statusbar, GTK_CAN_FOCUS);
 	gtk_box_pack_end (GTK_BOX (main_vbox), priv->statusbar, FALSE, FALSE, 0);
 	gtk_widget_show (priv->statusbar);
+
+	/* hack to get the same shadow as the status bar.. */
+	GtkShadowType shadow_type;
+	gtk_widget_style_get (GTK_WIDGET (priv->statusbar), "shadow-type", &shadow_type, NULL);
+
+	GtkWidget *frame = gtk_frame_new (NULL);
+	gtk_frame_set_shadow_type (GTK_FRAME (frame), shadow_type);
+	gtk_widget_show (frame);
+
+	gtk_box_pack_end (GTK_BOX (priv->statusbar), frame, FALSE, TRUE, 0);
+
+	hbox = gtk_hbox_new (FALSE, 0);
+	gtk_container_add (GTK_CONTAINER (frame), hbox);
+	gtk_box_set_spacing (GTK_BOX (hbox), 6);
+
+	priv->volume_label = gtk_label_new (_("Level:"));
+	gtk_box_pack_start (GTK_BOX (hbox), priv->volume_label, FALSE, TRUE, 0);
+
+	/* initialize priv->level */
+	align = gtk_aspect_frame_new ("", 0.0, 0.0, 20, FALSE);
+	gtk_frame_set_shadow_type (GTK_FRAME (align), GTK_SHADOW_NONE);
+	gtk_widget_show (align);
+	gtk_box_pack_start (GTK_BOX (hbox), align, FALSE, FALSE, 0);
+
+	priv->level = gtk_progress_bar_new ();
+	gtk_container_add (GTK_CONTAINER (align), priv->level);
+
+	gtk_widget_set_sensitive (window->priv->volume_label, FALSE);
+	gtk_widget_set_sensitive (window->priv->level, FALSE);
+
 	priv->status_message_cid = gtk_statusbar_get_context_id
 		(GTK_STATUSBAR (priv->statusbar), "status_message");
 	priv->tip_message_cid = gtk_statusbar_get_context_id
@@ -2311,7 +2499,7 @@ gsr_window_init (GSRWindow *window)
 
 	priv->len_secs = 0;
 	priv->get_length_attempts = 16;
-	priv->dirty = FALSE;
+	priv->dirty = TRUE;
 }
 
 static void
@@ -2324,6 +2512,7 @@ gsr_window_set_property (GObject      *object,
 	GSRWindowPrivate *priv;
 	struct stat buf;
 	char *title, *short_name;
+	char *utf8_name = NULL;
 	const char *ext;
 
 	window = GSR_WINDOW (object);
@@ -2332,7 +2521,9 @@ gsr_window_set_property (GObject      *object,
 	switch (prop_id) {
 	case PROP_LOCATION:
 		if (priv->filename != NULL) {
-			if (strcmp (g_value_get_string (value), priv->filename) == 0) {
+			if (g_value_get_string (value) == NULL)
+				return;
+			if (g_str_equal (g_value_get_string (value), priv->filename)) {
 				return;
 			}
 		}
@@ -2357,17 +2548,20 @@ gsr_window_set_property (GObject      *object,
 		else
 			window->priv->extension = NULL;
 
+		utf8_name = g_filename_to_utf8 (short_name, -1, NULL, NULL, NULL);
 		if (priv->name_label != NULL) {
-			gtk_label_set (GTK_LABEL (priv->name_label), short_name);
+			gtk_label_set (GTK_LABEL (priv->name_label), utf8_name);
 		}
 
 		if (recent_model) {
 			gsr_add_recent (priv->filename);
 		}
 
-		title = g_strdup_printf ("%s - Sound Recorder", short_name);
+		/*Translators: this is the window title, %s is the currently open file's name or Untitled*/
+		title = g_strdup_printf (_("%s - Sound Recorder"), utf8_name);
 		gtk_window_set_title (GTK_WINDOW (window), title);
 		g_free (title);
+		g_free (utf8_name);
 		g_free (short_name);
 
 		set_action_sensitive (window, "Play", window->priv->has_file ? TRUE : FALSE);
@@ -2514,7 +2708,7 @@ GtkWidget *
 gsr_window_new (const char *filename)
 {
 	GSRWindow *window;
-	struct stat buf;
+	char *template;
 
 	/* filename has been changed to be without extension */
 	window = g_object_new (GSR_TYPE_WINDOW, 
@@ -2522,15 +2716,16 @@ gsr_window_new (const char *filename)
 			       NULL);
         /* FIXME: check extension too */
 	window->priv->filename = g_strdup (filename);
-	if (stat (filename, &buf) == 0) {
+	if (g_file_test (filename, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR) != FALSE) {
 		window->priv->has_file = TRUE;
+		window->priv->dirty = FALSE;
 	} else {
 		window->priv->has_file = FALSE;
 	}
 
-	window->priv->record_filename = g_strdup_printf ("%s/gsr-record-%s-%d.XXXXXX",
-							 g_get_tmp_dir(), filename, getpid ());
-	window->priv->record_fd = g_mkstemp (window->priv->record_filename);
+	template = g_strdup_printf ("gsr-record-%s-%d.XXXXXX", filename, getpid ());
+	window->priv->record_fd = g_file_open_tmp (template, &window->priv->record_filename, NULL);
+	g_free (template);
 	close (window->priv->record_fd);
 
 	if (window->priv->has_file == FALSE) {
