@@ -23,7 +23,8 @@
 #include "config.h"
 #endif
 
-#include <gnome.h>
+#include <string.h>
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gconf/gconf-client.h>
 
@@ -41,6 +42,8 @@ enum {
   NUM_COLS
 };
 
+G_DEFINE_TYPE (GnomeVolumeControlPreferences, gnome_volume_control_preferences, GTK_TYPE_DIALOG)
+
 static void	gnome_volume_control_preferences_class_init	(GnomeVolumeControlPreferencesClass *klass);
 static void	gnome_volume_control_preferences_init	(GnomeVolumeControlPreferences *prefs);
 static void	gnome_volume_control_preferences_dispose (GObject *object);
@@ -61,35 +64,6 @@ static void	cb_gconf		(GConfClient     *client,
 					 GConfEntry      *entry,
 					 gpointer         userdata);
 
-static GtkNotebookClass *parent_class = NULL;
-
-GType
-gnome_volume_control_preferences_get_type (void)
-{
-  static GType gnome_volume_control_preferences_type = 0;
-
-  if (!gnome_volume_control_preferences_type) {
-    static const GTypeInfo gnome_volume_control_preferences_info = {
-      sizeof (GnomeVolumeControlPreferencesClass),
-      NULL,
-      NULL,
-      (GClassInitFunc) gnome_volume_control_preferences_class_init,
-      NULL,
-      NULL,
-      sizeof (GnomeVolumeControlPreferences),
-      0,
-      (GInstanceInitFunc) gnome_volume_control_preferences_init,
-      NULL
-    };
-
-    gnome_volume_control_preferences_type =
-	g_type_register_static (GTK_TYPE_DIALOG, 
-				"GnomeVolumeControlPreferences",
-				&gnome_volume_control_preferences_info, 0);
-  }
-
-  return gnome_volume_control_preferences_type;
-}
 
 static void
 gnome_volume_control_preferences_class_init (GnomeVolumeControlPreferencesClass *klass)
@@ -97,8 +71,6 @@ gnome_volume_control_preferences_class_init (GnomeVolumeControlPreferencesClass
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   GtkDialogClass *gtkdialog_class = (GtkDialogClass *) klass;
 
-  parent_class = g_type_class_ref (GTK_TYPE_DIALOG);
-
   gobject_class->dispose = gnome_volume_control_preferences_dispose;
   gtkdialog_class->response = gnome_volume_control_preferences_response;
 }
@@ -230,7 +202,7 @@ gnome_volume_control_preferences_dispose (GObject *object)
     prefs->mixer = NULL;
   }
 
-  G_OBJECT_CLASS (parent_class)->dispose (object);
+  G_OBJECT_CLASS (gnome_volume_control_preferences_parent_class)->dispose (object);
 }
 
 static void
@@ -246,8 +218,8 @@ gnome_volume_control_preferences_response (GtkDialog *dialog,
       break;
   }
 
-  if (((GtkDialogClass *) parent_class)->response)
-    ((GtkDialogClass *) parent_class)->response (dialog, response_id);
+  if (((GtkDialogClass *) gnome_volume_control_preferences_parent_class)->response)
+    ((GtkDialogClass *) gnome_volume_control_preferences_parent_class)->response (dialog, response_id);
 }
 
 /*
@@ -380,8 +352,7 @@ cb_gconf(GConfClient *client, guint connection_id,
   model = gtk_tree_view_get_model (GTK_TREE_VIEW(prefs->treeview));
   keybase = get_gconf_key (prefs->mixer, NULL);
 
-  if (!strncmp (gconf_entry_get_key (entry),
-		keybase, strlen (keybase)) &&
+  if (g_str_equal (gconf_entry_get_key (entry), keybase) &&
       (value = gconf_entry_get_value (entry)) != NULL &&
       (value->type == GCONF_VALUE_BOOL)) {
     active = gconf_value_get_bool (value); 
@@ -391,7 +362,7 @@ cb_gconf(GConfClient *client, guint connection_id,
       gtk_tree_model_get (model, &iter,
 			  COL_TRACK, &track,
 			  -1);
-      if (strcmp (track->label, gconf_entry_get_key (entry) + strlen (keybase))) {
+      if (g_str_equal (track->label, gconf_entry_get_key (entry) + strlen (keybase))) {
 	gtk_list_store_set( GTK_LIST_STORE(model), &iter, COL_ACTIVE, active, -1);
 	break ;
       }
