@@ -19,14 +19,21 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include "config.h"
-#include "libgnome/gnome-i18n.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+#include <string.h>
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
+#include <glade/glade-xml.h>
+#include <libgnomeui/gnome-help.h>
+#include <gst/gst.h>
+
 #include "gmp-util.h"
 #include "audio-profile-edit.h"
 #include "audio-profile.h"
 #include "audio-profile-private.h"
-#include <gtk/gtk.h>
-#include <glade/glade-xml.h>
 
 struct _GMAudioProfileEditPrivate
 {
@@ -36,16 +43,12 @@ struct _GMAudioProfileEditPrivate
   GtkWidget *content;
 };
 
-static GObject*
-gm_audio_profile_edit_constructor (GType type,
-                                   guint n_construct_properties,
-                                   GObjectConstructParam *construct_properties);
-
 static void	gm_audio_profile_edit_init		(GMAudioProfileEdit *edit);
 static void	gm_audio_profile_edit_class_init	(GMAudioProfileEditClass *klass);
-static void	gm_audio_profile_edit_finalize		(GObject *object);
-static void	gm_audio_profile_edit_destroy		(GtkObject *object);
+static void	gm_audio_profile_edit_dispose		(GObject *object);
 
+static void     gm_audio_profile_edit_response          (GtkDialog *dialog,
+			                                 int        id);
 static GtkWidget*
 		gm_audio_profile_edit_get_widget	(GMAudioProfileEdit *dialog,
 							 const char *widget_name);
@@ -64,92 +67,28 @@ static void	on_profile_changed			(GMAudioProfile *profile,
 							 const GMAudioSettingMask *mask,
 							 GMAudioProfileEdit *dialog);
 
-static gpointer parent_class;
-
-GType
-gm_audio_profile_edit_get_type (void)
-{
-  static GType object_type = 0;
-
-  g_type_init ();
-
-  if (!object_type)
-  {
-    static const GTypeInfo object_info =
-    {
-      sizeof (GMAudioProfileEditClass),
-      (GBaseInitFunc) NULL,
-      (GBaseFinalizeFunc) NULL,
-      (GClassInitFunc) gm_audio_profile_edit_class_init,
-      NULL,           /* class_finalize */
-      NULL,           /* class_data */
-      sizeof (GMAudioProfileEdit),
-      0,              /* n_preallocs */
-      (GInstanceInitFunc) gm_audio_profile_edit_init,
-    };
-
-    object_type = g_type_register_static (GTK_TYPE_DIALOG,
-                                          "GMAudioProfileEdit",
-                                          &object_info, 0);
-  }
-
-return object_type;
-}
-
-/* widget callbacks */
-
-//FIXME
-static void
-fix_button_align (GtkWidget *button)
-{
-  GtkWidget *child;
-
-  child = gtk_bin_get_child (GTK_BIN (button));
-
-  if (GTK_IS_ALIGNMENT (child))
-    g_object_set (G_OBJECT (child), "xalign", 0.0, NULL);
-  else if (GTK_IS_LABEL (child))
-    g_object_set (G_OBJECT (child), "xalign", 0.0, NULL);
-}
+G_DEFINE_TYPE (GMAudioProfileEdit, gm_audio_profile_edit, GTK_TYPE_DIALOG)
 
 /* ui callbacks */
 
-/* FIXME: do we need this ? */
-static void
-gap_remove (GtkWidget *widget, gpointer *data)
-{
-  gtk_container_remove (GTK_CONTAINER (data), widget);
-}
-
 /* initialize a dialog widget from the glade xml file */
 static void
 gm_audio_profile_edit_init (GMAudioProfileEdit *dialog)
 {
-  GladeXML *xml;
-  GtkWidget *vbox;
-
-  dialog->priv = g_new0 (GMAudioProfileEditPrivate, 1);
+  dialog->priv = G_TYPE_INSTANCE_GET_PRIVATE (dialog, GM_AUDIO_TYPE_PROFILE_EDIT, GMAudioProfileEditPrivate);
 }
 
 static void
 gm_audio_profile_edit_class_init (GMAudioProfileEditClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
+  GtkDialogClass *dialog_class = GTK_DIALOG_CLASS (klass);
 
-  parent_class = g_type_class_peek_parent (klass);
+  object_class->dispose = gm_audio_profile_edit_dispose;
 
-  /* FIXME: this code tries to override the constructor so it doesn't
-     actually create the gtkdialog yet ! */
-//  object_class->constructor = gm_audio_profile_edit_constructor;
-  object_class->finalize = gm_audio_profile_edit_finalize;
-}
+  dialog_class->response = gm_audio_profile_edit_response;
 
-static GObject*
-gm_audio_profile_edit_constructor (GType                  type,
-                              guint                  n_construct_properties,
-                              GObjectConstructParam *construct_properties)
-{
-  return NULL;
+  g_type_class_add_private (object_class, sizeof (GMAudioProfileEditPrivate));
 }
 
 static void
@@ -159,44 +98,44 @@ gm_audio_profile_edit_finalize (GObject *object)
 
   dialog = GM_AUDIO_PROFILE_EDIT (object);
 
-  g_free (dialog->priv);
-
-  G_OBJECT_CLASS (parent_class)->finalize (object);
+  G_OBJECT_CLASS (gm_audio_profile_edit_parent_class)->finalize (object);
 }
 
 /* ui callbacks */
 static void
-on_gm_audio_profile_edit_response (GMAudioProfileEdit *dialog,
-                              int        id,
-                              void      *data)
+gm_audio_profile_edit_response (GtkDialog *dialog,
+                                int        id)
 {
   if (id == GTK_RESPONSE_HELP)
     {
-      GError *err;
-      err = NULL;
-/*
-      gnome_help_display ("gnome-audio-profiles",
-                          "gnome-audio-profiles-profile-edit",
-                          &err);
-*/
+      GError *err = NULL;
+
+      gnome_help_display_on_screen ("gnome-audio-profiles",
+                                    "gnome-audio-profiles-profile-edit",
+				    gtk_widget_get_screen (GTK_WIDGET (dialog)),
+				    &err);
+
       if (err)
         {
           gmp_util_show_error_dialog  (GTK_WINDOW (dialog), NULL,
                                        _("There was an error displaying help: %s"), err->message);
           g_error_free (err);
         }
+
+      return;
     }
-  else
-    {
-      /* FIXME: hide or destroy ? */
-      gtk_widget_hide (GTK_WIDGET (dialog));
-    }
+      
+  /* FIXME: hide or destroy ? */
+  gtk_widget_hide (GTK_WIDGET (dialog));
 }
 
 static void
-on_gm_audio_profile_edit_destroy (GtkWidget *dialog, GMAudioProfile *profile)
+gm_audio_profile_edit_dispose (GObject *object)
 {
-  g_signal_handlers_disconnect_by_func (G_OBJECT (profile),
+  GMAudioProfileEdit *dialog = GM_AUDIO_PROFILE_EDIT (object);
+  GMAudioProfileEditPrivate *priv = dialog->priv;
+
+  g_signal_handlers_disconnect_by_func (priv->profile,
                                         G_CALLBACK (on_profile_changed),
                                         dialog);
 }
@@ -234,15 +173,13 @@ on_profile_name_changed (GtkWidget       *entry,
 }
 
 static void
-on_profile_description_changed (GtkWidget       *entry,
+on_profile_description_changed (GtkTextBuffer  *tb,
                                 GMAudioProfile *profile)
 {
   char *text;
 
-  text = gtk_editable_get_chars (GTK_EDITABLE (entry), 0, -1);
-
+  g_object_get (G_OBJECT (tb), "text", &text, NULL);
   gm_audio_profile_set_description (profile, text);
-
   g_free (text);
 }
 
@@ -287,6 +224,7 @@ gm_audio_profile_edit_new (GConfClient *conf, const char *id)
   GMAudioProfileEdit *dialog;
   GladeXML *xml;
   GtkWidget *w;
+  GtkTextBuffer *tb;
 
   /* get the dialog */
   xml = gmp_util_load_glade_file (GM_AUDIO_GLADE_FILE,
@@ -307,30 +245,20 @@ gm_audio_profile_edit_new (GConfClient *conf, const char *id)
   dialog->priv->xml = xml;
 
   /* save the GConf stuff and get the profile belonging to this id */
-  g_object_ref (G_OBJECT (conf));
-  dialog->priv->conf = conf;
+  dialog->priv->conf = g_object_ref (conf);
 
   dialog->priv->profile = gm_audio_profile_lookup (id);
   g_assert (dialog->priv->profile);
-  /* connect callbacks */
-  g_signal_connect (G_OBJECT (dialog),
-                    "response",
-                    G_CALLBACK (on_gm_audio_profile_edit_response),
-                    dialog);
-
-  g_signal_connect (G_OBJECT (dialog),
-                    "destroy",
-                    G_CALLBACK (on_gm_audio_profile_edit_destroy),
-                    dialog);
 
   /* autoconnect doesn't handle data pointers, sadly, so do by hand */
   w = glade_xml_get_widget (xml, "profile-name-entry");
   gm_audio_profile_edit_update_name (dialog, dialog->priv->profile);
   g_signal_connect (G_OBJECT (w), "changed",
                     G_CALLBACK (on_profile_name_changed), dialog->priv->profile);
-  w = glade_xml_get_widget (xml, "profile-description-entry");
+  w = glade_xml_get_widget (xml, "profile-description-textview");
   gm_audio_profile_edit_update_description (dialog, dialog->priv->profile);
-  g_signal_connect (G_OBJECT (w), "changed",
+  tb = gtk_text_view_get_buffer (GTK_TEXT_VIEW (w));
+  g_signal_connect (G_OBJECT (tb), "changed",
                     G_CALLBACK (on_profile_description_changed), dialog->priv->profile);
    w = glade_xml_get_widget (xml, "profile-pipeline-entry");
   gm_audio_profile_edit_update_pipeline (dialog, dialog->priv->profile);
@@ -346,6 +274,7 @@ gm_audio_profile_edit_new (GConfClient *conf, const char *id)
                     G_CALLBACK (on_profile_active_toggled), dialog->priv->profile);
 
 
+  gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
   gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
 
   /* connect to profile changes */
@@ -367,12 +296,30 @@ entry_set_text_if_changed (GtkEntry   *entry,
 {
   char *s;
 
-  GMP_DEBUG("entry_set_text_if_changed on entry %p with text %s\n", entry, text);
+  GST_DEBUG ("entry_set_text_if_changed on entry %p with text %s\n", entry, text);
   s = gtk_editable_get_chars (GTK_EDITABLE (entry), 0, -1);
-  GMP_DEBUG("got editable text %s\n", s);
+  GST_DEBUG ("got editable text %s\n", s);
   if (text && strcmp (s, text) != 0)
     gtk_entry_set_text (GTK_ENTRY (entry), text);
-  GMP_DEBUG("entry_set_text_if_changed: got %s\n", s);
+  GST_DEBUG ("entry_set_text_if_changed: got %s\n", s);
+
+  g_free (s);
+}
+
+static void
+textview_set_text_if_changed (GtkTextView *view, const char *text)
+{
+  char *s;
+  GtkTextBuffer *tb;
+
+  GST_DEBUG ("textview_set_text_if_changed on textview %p with text %s\n",
+	    view, text);
+  tb = gtk_text_view_get_buffer (view);
+  g_object_get (G_OBJECT (tb), "text", &s, NULL);
+  GST_DEBUG ("got textview text %s\n", s);
+  if (s && strcmp (s, text) != 0)
+    g_object_set (G_OBJECT (tb), "text", text, NULL);
+  GST_DEBUG ("textview_set_text_if_changed: got %s\n", s);
 
   g_free (s);
 }
@@ -386,7 +333,7 @@ gm_audio_profile_edit_update_name (GMAudioProfileEdit *dialog,
 
   s = g_strdup_printf (_("Editing profile \"%s\""),
                        gm_audio_profile_get_name (profile));
-  GMP_DEBUG("g_p_e_u_n: title %s\n", s);
+  GST_DEBUG ("g_p_e_u_n: title %s\n", s);
 
   gtk_window_set_title (GTK_WINDOW (dialog), s);
 
@@ -405,10 +352,10 @@ gm_audio_profile_edit_update_description (GMAudioProfileEdit *dialog,
 {
   GtkWidget *w;
 
-  w = gm_audio_profile_edit_get_widget (dialog, "profile-description-entry");
+  w = gm_audio_profile_edit_get_widget (dialog, "profile-description-textview");
   g_assert (GTK_IS_WIDGET (w));
 
-  entry_set_text_if_changed (GTK_ENTRY (w),
+  textview_set_text_if_changed (GTK_TEXT_VIEW (w),
                              gm_audio_profile_get_description (profile));
 }
 
